package com.boylesoftware.cb2.examples.addressbook.presentation;

import javax.servlet.ServletException;

import com.boylesoftware.cb2.presentation.servlet.PresentationElement;
import com.boylesoftware.cb2.presentation.servlet.ActionContext;


/**
 * Presentation element for the action confirmation component.
 *
 * @author Lev Himmelfarb
 * @version $Id: ConfirmationPE.java,v 1.1.1.1 2004/04/04 18:51:46 levahim Rel $
 */
public final class ConfirmationPE
    implements PresentationElement {


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element input parameters
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Type of action just taken on the contact record. Can be "saved"
     * or "deleted". Required PE input parameter.
     */
    private String action;

    /**
     * Last name of the contact affected. Required PE input parameter.
     */
    private String contactLastName;

    /**
     * First name of the contact affected. Optional PE input
     * parameter.
     */
    private String contactFirstName;


    /**
     * Sets type of action just taken on the contact record. Can be
     * "saved" or "deleted".
     *
     * @param action type of action just taken on the contact
     * record.
     */
    public void setAction(String action) {

        this.action = action;
    }

    /**
     * Gets type of action just taken on the contact record. Can be "saved"
     * or "deleted".
     *
     * @return type of action just taken on the contact record.
     */
    public String getAction() {

        return this.action;
    }

    /**
     * Sets last name of the contact affected.
     *
     * @param contactLastName last name of the contact affected.
     */
    public void setContactLastName(String contactLastName) {

        this.contactLastName = contactLastName;
    }

    /**
     * Gets last name of the contact affected.
     *
     * @return last name of the contact affected.
     */
    public String getContactLastName() {

        return this.contactLastName;
    }

    /**
     * Sets first name of the contact affected.
     *
     * @param contactFirstName first name of the contact affected.
     */
    public void setContactFirstName(String contactFirstName) {

        this.contactFirstName = contactFirstName;
    }

    /**
     * Gets first name of the contact affected.
     *
     * @return first name of the contact affected.
     */
    public String getContactFirstName() {

        return this.contactFirstName;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element data
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Message describing the action taken on the contect record.
     */
    private String actionMessage;

    /**
     * Full name of the contact affected.
     */
    private String contactName;


    /**
     * Gets message describing the action taken on the contect record.
     *
     * @return message describing the action taken on the contect
     * record.
     */
    public String getActionMessage() {

        return this.actionMessage;
    }

    /**
     * Gets full name of the contact affected.
     *
     * @return full name of the contact affected.
     */
    public String getContactName() {

        return this.contactName;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element interface
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Initialize presentation element.
     *
     * @param actionCtx the action context.
     *
     * @throws ServletException if invalid data is fed to the action.
     */
    public void init(ActionContext actionCtx)
        throws ServletException {

        if("saved".equals(this.action))
            this.actionMessage = "saved";
        else if("deleted".equals(this.action))
            this.actionMessage = "deleted";
        else
            throw new ServletException("Invalid action parameter.");

        this.contactName = this.contactLastName +
            (this.contactFirstName != null ? ", " + this.contactFirstName : "");
    }

    /**
     * Reset presentation element's input parameter fields.
     */
    public void reset() {

        this.action = null;
        this.contactLastName = null;
        this.contactFirstName = null;
    }
}
