package com.boylesoftware.cb2.examples.addressbook.presentation;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;
import com.boylesoftware.cb2.BLException;
import com.boylesoftware.cb2.presentation.servlet.PresentationElement;
import com.boylesoftware.cb2.presentation.servlet.ActionContext;

import com.boylesoftware.cb2.examples.addressbook.AddressBookBLO;
import com.boylesoftware.cb2.examples.addressbook.PersonDM;


/**
 * Form bean/presentation element for the contact details. As a
 * presentation element this form can prepopulate itself if
 * <code>personId</code> input parameter is set. It is used for both
 * updating existing records and creating new ones.
 *
 * @author Lev Himmelfarb
 * @version $Id: ContactDetailsFormPE.java,v 1.1.1.1 2004/04/04 18:51:46 levahim Rel $
 */
public final class ContactDetailsFormPE
    extends ActionForm
    implements PresentationElement {


    ///////////////////////////////////////////////////////////////////////////
    // PE input parameters
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Optional input parameter with person id to fetch and
     * prepopulate form fields. This is also a hidden form field, so
     * when we submit this form to the action, the action can
     * determine whether we are updating an existing contact, or
     * creating a new one, in which case this field will be
     * <code>null</code>.
     */
    private Integer personId;


    /**
     * Gets the person id hidden form field/input parameter.
     *
     * @return the personId value.
     */
    public Integer getPersonId() {

        return this.personId;
    }

    /**
     * Sets the person id hidden form field/input parameter.
     *
     * @param personId new personId value.
     */
    public void setPersonId(Integer personId) {

        this.personId = personId;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Form fields
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Hidden form field, which is set to "save" if the form is
     * submitted. If the form page is displayed without submission,
     * for example a contact details or add new contact link is
     * clicked, this field is not set, that is it is
     * <code>null</code>. The presentation element <code>init</code>
     * method uses this field to determine if the form should be
     * prepopulated (provided that <code>personId</code> is in the
     * request) or not. The form page may be redisplayed after form
     * submission if there were errors and the entered by the user
     * data cannot be accepted. In this case the <code>action</code>
     * field will be set to "save" and the <code>init</code> will not
     * perform form prepopulatio leaving data entered by the user
     * intact and to be corrected before retyring to submit the form
     * again.
     */
    private String action;

    /**
     * Contact's last name.
     */
    private String lastName;

    /**
     * Contact's first name.
     */
    private String firstName;

    /**
     * Contact's email.
     */
    private String email;

    /**
     * Whether the contact has not disabled receiving promotion
     * emails.
     */
    private boolean wantsSpam;

    /**
     * Additional comments for the contact.
     */
    private String comments;

    /**
     * Contact's home street address, line 1.
     */
    private String homeStreet1;

    /**
     * Contact's home street address, line 2.
     */
    private String homeStreet2;

    /**
     * Contact's home city.
     */
    private String homeCity;

    /**
     * Contact's home state.
     */
    private String homeState;

    /**
     * Contact's home zip code.
     */
    private String homeZip;

    /**
     * Contact's work street address, line 1.
     */
    private String workStreet1;

    /**
     * Contact's work street address, line 2.
     */
    private String workStreet2;

    /**
     * Contact's work city.
     */
    private String workCity;

    /**
     * Contact's work state.
     */
    private String workState;

    /**
     * Contact's work zip code.
     */
    private String workZip;

    /**
     * Contact's first telephone type.
     */
    private String phone1Type;

    /**
     * Contact's first telephone number.
     */
    private String phone1;

    /**
     * Contact's second telephone type.
     */
    private String phone2Type;

    /**
     * Contact's second telephone number.
     */
    private String phone2;

    /**
     * Contact's third telephone type.
     */
    private String phone3Type;

    /**
     * Contact's third telephone number.
     */
    private String phone3;

    /**
     * Contact's fourth telephone type.
     */
    private String phone4Type;

    /**
     * Contact's fourth telephone number.
     */
    private String phone4;

    /**
     * Contact's fifth telephone type.
     */
    private String phone5Type;

    /**
     * Contact's fifth telephone number.
     */
    private String phone5;


    /**
     * Sets requested action.
     *
     * @param action the requested action. May be "save" or
     * <code>null</code>.
     */
    public void setAction(String action) {

        this.action = action;
    }

    /**
     * Gets requested action.
     *
     * @return the requested action.
     */
    public String getAction() {

        return this.action;
    }

    /**
     * Sets contact's last name.
     *
     * @param lastName contact's last name.
     */
    public void setLastName(String lastName) {

        this.lastName = lastName;
    }

    /**
     * Gets contact's last name.
     *
     * @return contact's last name.
     */
    public String getLastName() {

        return this.lastName;
    }

    /**
     * Sets contact's first name.
     *
     * @param firstName contact's first name.
     */
    public void setFirstName(String firstName) {

        this.firstName = firstName;
    }

    /**
     * Gets contact's first name.
     *
     * @return contact's first name.
     */
    public String getFirstName() {

        return this.firstName;
    }

    /**
     * Sets contact's email.
     *
     * @param email contact's email.
     */
    public void setEmail(String email) {

        this.email = email;
    }

    /**
     * Gets contact's email.
     *
     * @return contact's email.
     */
    public String getEmail() {

        return this.email;
    }

    /**
     * Sets whether the contact has not disabled receiving promotion
     * emails.
     *
     * @param wantsSpam whether the contact has not disabled receiving
     * promotion emails.
     */
    public void setWantsSpam(boolean wantsSpam) {

        this.wantsSpam = wantsSpam;
    }

    /**
     * Gets whether the contact has not disabled receiving promotion
     * emails.
     *
     * @return whether the contact has not disabled receiving
     * promotion emails.
     */
    public boolean getWantsSpam() {

        return this.wantsSpam;
    }

    /**
     * Sets additional comments for the contact.
     *
     * @param comments additional comments for the contact.
     */
    public void setComments(String comments) {

        this.comments = comments;
    }

    /**
     * Gets additional comments for the contact.
     *
     * @return additional comments for the contact.
     */
    public String getComments() {

        return this.comments;
    }

    /**
     * Sets contact's home street address, line 1.
     *
     * @param homeStreet1 contact's home street address, line 1.
     */
    public void setHomeStreet1(String homeStreet1) {

        this.homeStreet1 = homeStreet1;
    }

    /**
     * Gets contact's home street address, line 1.
     *
     * @return contact's home street address, line 1.
     */
    public String getHomeStreet1() {

        return this.homeStreet1;
    }

    /**
     * Sets contact's home street address, line 2.
     *
     * @param homeStreet2 contact's home street address, line 2.
     */
    public void setHomeStreet2(String homeStreet2) {

        this.homeStreet2 = homeStreet2;
    }

    /**
     * Gets contact's home street address, line 2.
     *
     * @return contact's home street address, line 2.
     */
    public String getHomeStreet2() {

        return this.homeStreet2;
    }

    /**
     * Sets contact's home city.
     *
     * @param homeCity contact's home city.
     */
    public void setHomeCity(String homeCity) {

        this.homeCity = homeCity;
    }

    /**
     * Gets contact's home city.
     *
     * @return contact's home city.
     */
    public String getHomeCity() {

        return this.homeCity;
    }

    /**
     * Sets contact's home state.
     *
     * @param homeState contact's home state.
     */
    public void setHomeState(String homeState) {

        this.homeState = homeState;
    }

    /**
     * Gets contact's home state.
     *
     * @return contact's home state.
     */
    public String getHomeState() {

        return this.homeState;
    }

    /**
     * Sets contact's home zip code.
     *
     * @param homeZip contact's home zip code.
     */
    public void setHomeZip(String homeZip) {

        this.homeZip = homeZip;
    }

    /**
     * Gets contact's home zip code.
     *
     * @return contact's home zip code.
     */
    public String getHomeZip() {

        return this.homeZip;
    }

    /**
     * Sets contact's work street address, line 1.
     *
     * @param workStreet1 contact's work street address, line 1.
     */
    public void setWorkStreet1(String workStreet1) {

        this.workStreet1 = workStreet1;
    }

    /**
     * Gets contact's work street address, line 1.
     *
     * @return contact's work street address, line 1.
     */
    public String getWorkStreet1() {

        return this.workStreet1;
    }

    /**
     * Sets contact's work street address, line 2.
     *
     * @param workStreet2 contact's work street address, line 2.
     */
    public void setWorkStreet2(String workStreet2) {

        this.workStreet2 = workStreet2;
    }

    /**
     * Gets contact's work street address, line 2.
     *
     * @return contact's work street address, line 2.
     */
    public String getWorkStreet2() {

        return this.workStreet2;
    }

    /**
     * Sets contact's work city.
     *
     * @param workCity contact's work city.
     */
    public void setWorkCity(String workCity) {

        this.workCity = workCity;
    }

    /**
     * Gets contact's work city.
     *
     * @return contact's work city.
     */
    public String getWorkCity() {

        return this.workCity;
    }

    /**
     * Sets contact's work state.
     *
     * @param workState contact's work state.
     */
    public void setWorkState(String workState) {

        this.workState = workState;
    }

    /**
     * Gets contact's work state.
     *
     * @return contact's work state.
     */
    public String getWorkState() {

        return this.workState;
    }

    /**
     * Sets contact's work zip code.
     *
     * @param workZip contact's work zip code.
     */
    public void setWorkZip(String workZip) {

        this.workZip = workZip;
    }

    /**
     * Gets contact's work zip code.
     *
     * @return contact's work zip code.
     */
    public String getWorkZip() {

        return this.workZip;
    }

    /**
     * Sets contact's first telephone type.
     *
     * @param phone1Type contact's first telephone type.
     */
    public void setPhone1Type(String phone1Type) {

        this.phone1Type = phone1Type;
    }

    /**
     * Gets contact's first telephone type.
     *
     * @return contact's first telephone type.
     */
    public String getPhone1Type() {

        return this.phone1Type;
    }

    /**
     * Sets contact's first telephone number.
     *
     * @param phone1 contact's first telephone number.
     */
    public void setPhone1(String phone1) {

        this.phone1 = phone1;
    }

    /**
     * Gets contact's first telephone number.
     *
     * @return contact's first telephone number.
     */
    public String getPhone1() {

        return this.phone1;
    }

    /**
     * Sets contact's second telephone type.
     *
     * @param phone2Type contact's second telephone type.
     */
    public void setPhone2Type(String phone2Type) {

        this.phone2Type = phone2Type;
    }

    /**
     * Gets contact's second telephone type.
     *
     * @return contact's second telephone type.
     */
    public String getPhone2Type() {

        return this.phone2Type;
    }

    /**
     * Sets contact's second telephone number.
     *
     * @param phone2 contact's second telephone number.
     */
    public void setPhone2(String phone2) {

        this.phone2 = phone2;
    }

    /**
     * Gets contact's second telephone number.
     *
     * @return contact's second telephone number.
     */
    public String getPhone2() {

        return this.phone2;
    }

    /**
     * Sets contact's third telephone type.
     *
     * @param phone3Type contact's third telephone type.
     */
    public void setPhone3Type(String phone3Type) {

        this.phone3Type = phone3Type;
    }

    /**
     * Gets contact's third telephone type.
     *
     * @return contact's third telephone type.
     */
    public String getPhone3Type() {

        return this.phone3Type;
    }

    /**
     * Sets contact's third telephone number.
     *
     * @param phone3 contact's third telephone number.
     */
    public void setPhone3(String phone3) {

        this.phone3 = phone3;
    }

    /**
     * Gets contact's third telephone number.
     *
     * @return contact's third telephone number.
     */
    public String getPhone3() {

        return this.phone3;
    }

    /**
     * Sets contact's fourth telephone type.
     *
     * @param phone4Type contact's fourth telephone type.
     */
    public void setPhone4Type(String phone4Type) {

        this.phone4Type = phone4Type;
    }

    /**
     * Gets contact's fourth telephone type.
     *
     * @return contact's fourth telephone type.
     */
    public String getPhone4Type() {

        return this.phone4Type;
    }

    /**
     * Sets contact's fourth telephone number.
     *
     * @param phone4 contact's fourth telephone number.
     */
    public void setPhone4(String phone4) {

        this.phone4 = phone4;
    }

    /**
     * Gets contact's fourth telephone number.
     *
     * @return contact's fourth telephone number.
     */
    public String getPhone4() {

        return this.phone4;
    }

    /**
     * Sets contact's fifth telephone type.
     *
     * @param phone5Type contact's fifth telephone type.
     */
    public void setPhone5Type(String phone5Type) {

        this.phone5Type = phone5Type;
    }

    /**
     * Gets contact's fifth telephone type.
     *
     * @return contact's fifth telephone type.
     */
    public String getPhone5Type() {

        return this.phone5Type;
    }

    /**
     * Sets contact's fifth telephone number.
     *
     * @param phone5 contact's fifth telephone number.
     */
    public void setPhone5(String phone5) {

        this.phone5 = phone5;
    }

    /**
     * Gets contact's fifth telephone number.
     *
     * @return contact's fifth telephone number.
     */
    public String getPhone5() {

        return this.phone5;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Other PE fields
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Labels for the phone type dropdown.
     */
    private final static String [] phoneTypeLabels = {
        "Home", "Work", "Mobile", "Fax", "Pager", "Other"
    };

    /**
     * Values for the phone type dropdown.
     */
    private final static String [] phoneTypeCodes = {
        "HOM", "WRK", "MOB", "FAX", "PAG", "OTH"
    };


    /**
     * Gets labels for the phone type dropdown.
     *
     * @return the list of lables.
     */
    public String [] getPhoneTypeLabels() {

        return phoneTypeLabels;
    }

    /**
     * Gets values for the phone type dropdown.
     *
     * @return the list of codes.
     */
    public String [] getPhoneTypeCodes() {

        return phoneTypeCodes;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element interface
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Initialize presentation element.
     *
     * @param actionCtx the action context.
     *
     * @throws BLException if an unexpected error happens in the
     * business level.
     */
    public void init(ActionContext actionCtx)
        throws BLException {

        // check if we need to prepopulate the form and do it if so
        if(!"save".equals(this.action) && this.personId != null) {

            AddressBookBLO abBlo = (AddressBookBLO)actionCtx.getBLO("addressBook");

            PersonDM pnDm = abBlo.getPersonDetails(this.personId.intValue());
            this.lastName = pnDm.lastName;
            this.firstName = pnDm.firstName;
            this.email = pnDm.email;
            this.wantsSpam = pnDm.wantsSpam;
            this.comments = pnDm.comments;
            if(pnDm.homeAddress != null) {
                this.homeStreet1 = pnDm.homeAddress.street1;
                this.homeStreet2 = pnDm.homeAddress.street2;
                this.homeCity = pnDm.homeAddress.city;
                this.homeState = pnDm.homeAddress.state;
                this.homeZip = pnDm.homeAddress.zip;
            }
            if(pnDm.workAddress != null) {
                this.workStreet1 = pnDm.workAddress.street1;
                this.workStreet2 = pnDm.workAddress.street2;
                this.workCity = pnDm.workAddress.city;
                this.workState = pnDm.workAddress.state;
                this.workZip = pnDm.workAddress.zip;
            }
            for(int i = 0; i < pnDm.phones.length; i++) {
                switch(i) {
                case 0:
                    this.phone1Type = pnDm.phones[0].type;
                    this.phone1 = this.formatPhone(pnDm.phones[0].phone);
                    break;
                case 1:
                    this.phone2Type = pnDm.phones[1].type;
                    this.phone2 = this.formatPhone(pnDm.phones[1].phone);
                    break;
                case 2:
                    this.phone3Type = pnDm.phones[2].type;
                    this.phone3 = this.formatPhone(pnDm.phones[2].phone);
                    break;
                case 3:
                    this.phone4Type = pnDm.phones[3].type;
                    this.phone4 = this.formatPhone(pnDm.phones[3].phone);
                    break;
                case 4:
                    this.phone5Type = pnDm.phones[4].type;
                    this.phone5 = this.formatPhone(pnDm.phones[4].phone);
                    break;
                }
            }
        }
    }

    /**
     * Reset presentation element's internal fields.
     */
    public void reset() {

        this.personId = null; // if not in the request let it be null
    }

    /**
     * Formats phone number from the DM format to UI format, that is
     * from a raw sequence of digits to (XXX) XXX-XXXX.
     *
     * @param phone phone number from DM, can be <code>null</code>.
     *
     * @return formatted phone number or <code>null</code>.
     */
    private String formatPhone(String phone) {

        if(phone == null)
            return null;

        return "(" + phone.substring(0, 3) + ") " +
            phone.substring(3, 6) + "-" + phone.substring(6);
    }


    ///////////////////////////////////////////////////////////////////////////
    // Form bean interface
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Reset form bean's fields to their default values.
     */
    public void reset(ActionMapping mapping, HttpServletRequest request) {

        this.personId = null; // it is also a form field
        this.action = null;
        this.lastName = null;
        this.firstName = null;
        this.email = null;
        this.wantsSpam = false;
        this.comments = null;
        this.homeStreet1 = null;
        this.homeStreet2 = null;
        this.homeCity = null;
        this.homeState = null;
        this.homeZip = null;
        this.workStreet1 = null;
        this.workStreet2 = null;
        this.workCity = null;
        this.workState = null;
        this.workZip = null;
        this.phone1Type = null;
        this.phone1 = null;
        this.phone2Type = null;
        this.phone2 = null;
        this.phone3Type = null;
        this.phone3 = null;
        this.phone4Type = null;
        this.phone4 = null;
        this.phone5Type = null;
        this.phone5 = null;
    }
}
