package com.boylesoftware.cb2.examples.addressbook.presentation;

import javax.servlet.http.HttpServletRequest;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionMapping;
import com.boylesoftware.cb2.presentation.servlet.PresentationElement;
import com.boylesoftware.cb2.presentation.servlet.ActionContext;


/**
 * Form bean/presentation element for the contacts search form.
 *
 * @author Lev Himmelfarb
 * @version $Id: ContactsSearchFormPE.java,v 1.1.1.1 2004/04/04 18:51:45 levahim Rel $
 */
public final class ContactsSearchFormPE
    extends ActionForm
    implements PresentationElement {


    ///////////////////////////////////////////////////////////////////////////
    // Form fields
    ///////////////////////////////////////////////////////////////////////////

    /**
     * "Do Search" hidden field that tells the
     * <code>ContactsListPE</code> whether to perform contacts search
     * on this form submission or to fetch all contacts in the
     * database. If the <code>ContactsListPE</code> is invoked as a
     * result of this form submission then this field is
     * <code>true</code> (as defined in the component's JSP) and the
     * <code>ContactsListPE</code> performs contacts search basing on
     * values in the fields of this form. If the page is displayed
     * without form submission, that is by clicking a link or by
     * entering the URL, this field has its default value of
     * <code>false</code> and <code>ContactsListPE</code> fetches all
     * contacts from the database to display them in the component.
     */
    private boolean doSearch;

    /**
     * "Last Name" form input field.
     */
    private String lastName;

    /**
     * "First Name" form input field.
     */
    private String firstName;

    /**
     * "City" form input field.
     */
    private String city;

    /**
     * "State" form input field.
     */
    private String state;


    /**
     * Gets the "Do Search" form hidden field.
     *
     * @return the field value.
     */
    public boolean getDoSearch() {

        return this.doSearch;
    }

    /**
     * Sets the "Do Search" form hidden field.
     *
     * @param doSearch new field value.
     */
    public void setDoSearch(boolean doSearch) {

        this.doSearch = doSearch;
    }

    /**
     * Gets the "Last Name" form input field.
     *
     * @return the field value.
     */
    public String getLastName() {

        return this.lastName;
    }

    /**
     * Sets the "Last Name" form input field.
     *
     * @param lastName new field value.
     */
    public void setLastName(String lastName) {

        this.lastName = lastName;
    }

    /**
     * Gets the "First Name" form input field.
     *
     * @return the field value.
     */
    public String getFirstName() {

        return this.firstName;
    }

    /**
     * Sets the "First Name" form input field.
     *
     * @param firstName new field value.
     */
    public void setFirstName(String firstName) {

        this.firstName = firstName;
    }

    /**
     * Gets the "City" form input field.
     *
     * @return the field value.
     */
    public String getCity() {

        return this.city;
    }

    /**
     * Sets the "City" form input field.
     *
     * @param city new field value.
     */
    public void setCity(String city) {

        this.city = city;
    }

    /**
     * Gets the "State" form input field.
     *
     * @return the field value.
     */
    public String getState() {

        return this.state;
    }

    /**
     * Sets the "State" form input field.
     *
     * @param state new field value.
     */
    public void setState(String state) {

        this.state = state;
    }


    ///////////////////////////////////////////////////////////////////////////
    // Presentation element interface
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Initialize presentation element.
     *
     * @param actionCtx the action context.
     */
    public void init(ActionContext actionCtx) {}

    /**
     * Reset presentation element's internal fields. It is not used in
     * this class since the presentation element represented by this
     * class has no input parameters. The method is still implemented
     * because it part of the interface.
     */
    public void reset() {}


    ///////////////////////////////////////////////////////////////////////////
    // Form bean interface
    ///////////////////////////////////////////////////////////////////////////

    /**
     * Reset form bean's fields to their default values.
     */
    public void reset(ActionMapping mapping, HttpServletRequest request) {

        this.doSearch = false; // default value for doSearch if it is not present in the request
        this.lastName = null;
        this.firstName = null;
        this.city = null;
        this.state = null;
    }
}
