package com.boylesoftware.cb2.examples.addressbook.presentation;

import java.util.List;
import java.util.ArrayList;
import java.util.regex.Pattern;

import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMessage;
import com.boylesoftware.cb2.BLErrors;
import com.boylesoftware.cb2.BLException;
import com.boylesoftware.cb2.presentation.servlet.CB2Action;
import com.boylesoftware.cb2.presentation.servlet.ActionContext;
import com.boylesoftware.cb2.util.TextUtils;

import com.boylesoftware.cb2.examples.addressbook.AddressBookBLO;
import com.boylesoftware.cb2.examples.addressbook.PersonDM;
import com.boylesoftware.cb2.examples.addressbook.AddressDM;
import com.boylesoftware.cb2.examples.addressbook.PhoneDM;


/**
 * This action saves contact details by either creating a new record
 * or updating an existing one. It expects a
 * <code>ContactDetailsFormPE</code> at its input.
 *
 * @author Lev Himmelfarb
 * @version $Id: SaveContactDetailsAction.java,v 1.1.1.1 2004/04/04 18:51:46 levahim Rel $
 */
public final class SaveContactDetailsAction
    extends CB2Action {

    /**
     * RE pattern for validating zip codes.
     */
    private static final Pattern ZIP_PATTERN =
        Pattern.compile("^\\d{5}$");


    /**
     * Perform the action.
     *
     * @param actionCtx the action context.
     *
     * @throws BLException if an unexpected error happens in the
     * business level.
     */
    protected ActionForward execute(ActionContext actionCtx)
        throws BLException {

        // get the form
        ContactDetailsFormPE form = (ContactDetailsFormPE)actionCtx.getForm();

        // prepare errors object
        ActionErrors errors = new ActionErrors();

        // build the DM
        PersonDM pnDm = new PersonDM();
        pnDm.personId = (form.getPersonId() != null ? form.getPersonId().intValue() : 0);
        if((pnDm.lastName = TextUtils.getNullIfEmpty(form.getLastName())) == null)
            errors.add("lastName", new ActionMessage("error.lastNameRequired"));
        pnDm.firstName = TextUtils.getNullIfEmpty(form.getFirstName());
        if(TextUtils.isEmpty(form.getEmail()) || TextUtils.isValidEmailAddress(form.getEmail().trim()))
            pnDm.email = form.getEmail().trim();
        else
            errors.add("email", new ActionMessage("error.invalidEmail"));
        pnDm.wantsSpam = form.getWantsSpam();
        pnDm.comments = TextUtils.getNullIfEmpty(form.getComments());
        if(!(TextUtils.isEmpty(form.getHomeStreet1()) &&
             TextUtils.isEmpty(form.getHomeStreet2()) &&
             TextUtils.isEmpty(form.getHomeCity()) &&
             TextUtils.isEmpty(form.getHomeState()) &&
             TextUtils.isEmpty(form.getHomeZip()))) {
            pnDm.homeAddress = new AddressDM();
            pnDm.homeAddress.street1 = TextUtils.getNullIfEmpty(form.getHomeStreet1());
            pnDm.homeAddress.street2 = TextUtils.getNullIfEmpty(form.getHomeStreet2());
            pnDm.homeAddress.city = TextUtils.getNullIfEmpty(form.getHomeCity());
            if(TextUtils.isEmpty(form.getHomeState()) || (form.getHomeState().trim().length() == 2))
                pnDm.homeAddress.state = form.getHomeState().trim().toUpperCase();
            else
                errors.add("homeState", new ActionMessage("error.invalidState"));
            if(TextUtils.isEmpty(form.getHomeZip()) || ZIP_PATTERN.matcher(form.getHomeZip().trim()).matches())
                pnDm.homeAddress.zip = form.getHomeZip().trim();
            else
                errors.add("homeZip", new ActionMessage("error.invalidZip"));
        }
        if(!(TextUtils.isEmpty(form.getWorkStreet1()) &&
             TextUtils.isEmpty(form.getWorkStreet2()) &&
             TextUtils.isEmpty(form.getWorkCity()) &&
             TextUtils.isEmpty(form.getWorkState()) &&
             TextUtils.isEmpty(form.getWorkZip()))) {
            pnDm.workAddress = new AddressDM();
            pnDm.workAddress.street1 = TextUtils.getNullIfEmpty(form.getWorkStreet1());
            pnDm.workAddress.street2 = TextUtils.getNullIfEmpty(form.getWorkStreet2());
            pnDm.workAddress.city = TextUtils.getNullIfEmpty(form.getWorkCity());
            if(TextUtils.isEmpty(form.getWorkState()) || (form.getWorkState().trim().length() == 2))
                pnDm.workAddress.state = form.getWorkState().trim().toUpperCase();
            else
                errors.add("workState", new ActionMessage("error.invalidState"));
            if(TextUtils.isEmpty(form.getWorkZip()) || ZIP_PATTERN.matcher(form.getWorkZip().trim()).matches())
                pnDm.workAddress.zip = form.getWorkZip().trim();
            else
                errors.add("workZip", new ActionMessage("error.invalidZip"));
        }
        List phones = new ArrayList(5);
        if(!TextUtils.isEmpty(form.getPhone1())) {
            PhoneDM phDm = new PhoneDM();
            phDm.type = TextUtils.getNullIfEmpty(form.getPhone1Type());
            if(!TextUtils.isEmpty(form.getPhone1()) &&
               ((phDm.phone = this.parsePhone(form.getPhone1())) == null))
                errors.add("phone1", new ActionMessage("error.invalidPhone"));
            phones.add(phDm);
        }
        if(!TextUtils.isEmpty(form.getPhone2())) {
            PhoneDM phDm = new PhoneDM();
            phDm.type = TextUtils.getNullIfEmpty(form.getPhone2Type());
            if(!TextUtils.isEmpty(form.getPhone2()) &&
               ((phDm.phone = this.parsePhone(form.getPhone2())) == null))
                errors.add("phone2", new ActionMessage("error.invalidPhone"));
            phones.add(phDm);
        }
        if(!TextUtils.isEmpty(form.getPhone3())) {
            PhoneDM phDm = new PhoneDM();
            phDm.type = TextUtils.getNullIfEmpty(form.getPhone3Type());
            if(!TextUtils.isEmpty(form.getPhone3()) &&
               ((phDm.phone = this.parsePhone(form.getPhone3())) == null))
                errors.add("phone3", new ActionMessage("error.invalidPhone"));
            phones.add(phDm);
        }
        if(!TextUtils.isEmpty(form.getPhone4())) {
            PhoneDM phDm = new PhoneDM();
            phDm.type = TextUtils.getNullIfEmpty(form.getPhone4Type());
            if(!TextUtils.isEmpty(form.getPhone4()) &&
               ((phDm.phone = this.parsePhone(form.getPhone4())) == null))
                errors.add("phone4", new ActionMessage("error.invalidPhone"));
            phones.add(phDm);
        }
        if(!TextUtils.isEmpty(form.getPhone5())) {
            PhoneDM phDm = new PhoneDM();
            phDm.type = TextUtils.getNullIfEmpty(form.getPhone5Type());
            if(!TextUtils.isEmpty(form.getPhone5()) &&
               ((phDm.phone = this.parsePhone(form.getPhone5())) == null))
                errors.add("phone5", new ActionMessage("error.invalidPhone"));
            phones.add(phDm);
        }
        pnDm.phones = (PhoneDM [])phones.toArray(new PhoneDM[phones.size()]);

        // check if there where errors
        if(!errors.isEmpty()) {

            actionCtx.saveErrors(errors);

            return actionCtx.findForward("failure");
        }

        // submit data to the business level
        AddressBookBLO abBlo = (AddressBookBLO)actionCtx.getBLO("addressBook");
        if(!abBlo.savePersonDetails(pnDm)) {
            BLErrors blErrors = abBlo.getLastErrors();

            if(blErrors.containsError(AddressBookBLO.ERROR_NAME_EXISTS)) {
                errors.add("lastName", new ActionMessage("error.nameExists"));
                errors.add("firstName", new ActionMessage("error.nameExists"));
            } else
                errors.add(ActionErrors.GLOBAL_MESSAGE, new ActionMessage("error.unknown"));

            actionCtx.saveErrors(errors);

            return actionCtx.findForward("failure");
        }

        // set parameters for the forward
        actionCtx.setAttribute("contactLastName", pnDm.lastName, ActionContext.REQUEST_SCOPE);
        if(pnDm.firstName != null)
            actionCtx.setAttribute("contactFirstName", pnDm.firstName, ActionContext.REQUEST_SCOPE);

        // all done, return success
        return actionCtx.findForward("success");
    }

    /**
     * Tries to parse phone number and makes it suitable for saving in
     * the database (only digits).
     *
     * @param phone phone number as entered by the user, cannot be
     * <code>null</code>.
     *
     * @return ten digits from the phone, or <code>null</code> if not
     * valid.
     */
    private String parsePhone(String phone) {

        StringBuffer res = new StringBuffer(10);
        for(int i = 0; i < phone.length(); i++) {
            char c = phone.charAt(i);
            if(Character.isDigit(c))
                res.append(c);
        }

        return (res.length() == 10 ? res.toString() : null);
    }
}
