--
-- Creates database schema for the sample application.
-- To be used with MySQL like this:
--
--   mysql -h <host> -D <database> -u <user> -p<password> < schema_mysql.sql
--
-- author Lev Himmelfarb
-- version $Id: schema_mysql.sql,v 1.1.1.1 2004/04/04 18:51:46 levahim Rel $
--

drop table if exists Phone;
drop table if exists Person;
drop table if exists Address;


create table Address (
    addressId int unsigned not null auto_increment,
    street1 varchar(50),
    street2 varchar(50),
    city varchar(50),
    state char(2),
    zip char(5),
    constraint pk_Address
	primary key (addressId)
);

create table Person (
    personId int unsigned not null auto_increment,
    lastName varchar(50) not null,
    firstName varchar(50),
    homeAddressId int unsigned,
    workAddressId int unsigned,
    email varchar(30),
    wantsSpam bit not null,
    comments text,
    constraint pk_Person
	primary key (personId),
    constraint fk_Person_homeAddressId
	foreign key (homeAddressId) references Address (addressId),
    constraint fk_Person_workAddressId
	foreign key (workAddressId) references Address (addressId)
);

create table Phone (
    personId int unsigned not null,
    type char(3) not null,
    phone char(10) not null,
    constraint fk_Phone_personId
	foreign key (personId) references Person (personId),
    constraint ck_Phone_type
	check (type in ('HOM', 'WRK', 'MOB', 'FAX', 'PAG', 'OTH'))
);
