--
-- Creates database schema for the sample application.
-- To be used with Sybase ASE like this:
--
--   isql -S <server> -D <database> -U <user> -P <password> -i schema_sybase.sql
--
-- author Lev Himmelfarb
-- version $Id: schema_sybase.sql,v 1.1 2004/06/08 20:41:38 levahim Exp $
--

drop table Phone
drop table Person
drop table Address
go


create table Address (
    addressId numeric(6) identity not null,
    street1 varchar(50) null,
    street2 varchar(50) null,
    city varchar(50) null,
    state char(2) null,
    zip char(5) null,
    constraint pk_Address
	primary key (addressId)
)
go

create table Person (
    personId numeric(6) identity not null,
    lastName varchar(50) not null,
    firstName varchar(50) null,
    homeAddressId numeric(6) null,
    workAddressId numeric(6) null,
    email varchar(30) null,
    wantsSpam bit not null,
    comments text null,
    constraint pk_Person
	primary key (personId),
    constraint fk_Person_homeAddressId
	foreign key (homeAddressId) references Address (addressId),
    constraint fk_Person_workAddressId
	foreign key (workAddressId) references Address (addressId)
)
go

create table Phone (
    personId numeric(6) not null,
    type char(3) not null,
    phone char(10) not null,
    constraint fk_Phone_personId
	foreign key (personId) references Person (personId),
    constraint ck_Phone_type
	check (type in ('HOM', 'WRK', 'MOB', 'FAX', 'PAG', 'OTH'))
)
go
